DROP PROCEDURE IF EXISTS `copy_for_month`;
%QUERY_SEPARATOR%
CREATE PROCEDURE `copy_for_month`(
    IN drop_first BOOLEAN, IN year INT, IN month INT,
    IN base_table_name VARCHAR(50), IN temp_table_name VARCHAR(50))
BEGIN
  DECLARE new_table_name   VARCHAR(50);
  
  DECLARE min_date VARCHAR(10);
  DECLARE max_date VARCHAR(10);
  
  -- BUILD CORRECT DATE FORMAT yyyy-mm-dd
  IF month < 10
    THEN
      SET min_date = concat(year, "-0", month, "-01");
      SET new_table_name = concat(base_table_name, '_', year, "_0", month);
    ELSE
      SET min_date = concat(year, "-", month, "-01");
      SET new_table_name = concat(base_table_name, '_', year, "_", month);
  END IF;
  
  SELECT concat(CURDATE(), " ", CURTIME(),
                ": Moving telegrams for ", year, "-", month, " to ", new_table_name) AS "Action";
  
  CALL create_table_tele_history(drop_first, new_table_name);
  
  SELECT LAST_DAY(concat(year, '-', month, '-01')) INTO max_date;
  
  -- MOVE DATA FROM LIVE TABLE INTO MONTH ARCHIVE TABLE
  SET @query = concat(
    'INSERT INTO ', new_table_name, 
    ' SELECT NULL, system_id, sernr, ga1, data, tid, pid, ep_time, last_time, src_type, src_addr, rt_counter',
    ' FROM ', temp_table_name, 
    ' WHERE last_time >= "', min_date, ' 00:00:00"',
    ' AND last_time <= "', max_date, ' 23:59:59"');
    
  /* SELECT @query; */
  
  PREPARE sqlquery FROM @query;
  EXECUTE sqlquery;
  DEALLOCATE PREPARE sqlquery;
END;
%QUERY_SEPARATOR%

DROP PROCEDURE IF EXISTS `sort_into_archive_tables`;
%QUERY_SEPARATOR%
CREATE PROCEDURE `sort_into_archive_tables`(IN remove_archives BOOLEAN, IN live_table_name VARCHAR(50))
BEGIN
  DECLARE min_year   INT;
  DECLARE min_month  INT;
  DECLARE max_year   INT;
  DECLARE max_month  INT;
  
  SELECT concat(CURDATE(), " ", CURTIME(), ": Starting sort") AS "Action";
  
  SET @temp_table_name = concat(live_table_name, '_temp');
  SET @new_table_name = concat(live_table_name, '_new');
  
  -- CREATE NEW TABLE AND REPLACE live_table_name => temp_table_name AND new_table_name => live_table_name
  CALL create_table_tele_history(1, @new_table_name);
  
  SET @query = CONCAT('RENAME TABLE `', live_table_name, '` TO `' ,@temp_table_name, '`,',
                                  ' `', @new_table_name, '` TO `' ,live_table_name,'`');
  PREPARE sqlquery FROM @query;
  EXECUTE sqlquery;
  DEALLOCATE PREPARE sqlquery;
  
  -- SELECT MINIMUM AND MAXIMUM DATE FROM LIVE TABLE
  SET @query = concat('SELECT MIN(last_time), MAX(last_time) INTO @min_time, @max_time FROM ', @temp_table_name);
  PREPARE sqlquery FROM @query;
  EXECUTE sqlquery;
  DEALLOCATE PREPARE sqlquery;

  -- SPLIT DATES INTO YEAR AND MONTH
  SELECT YEAR(@min_time), MONTH(@min_time) INTO min_year, min_month;
  SELECT YEAR(@max_time), MONTH(@max_time) INTO max_year, max_month;
  
  -- FOR EVERY MONTH IN THIS TIMERANGE CALL copy_for_month
  WHILE min_year <= max_year DO
    IF min_year = max_year
      THEN
        WHILE min_month <= max_month DO
          CALL copy_for_month(remove_archives, min_year, min_month, live_table_name, @temp_table_name);
          SET min_month = min_month + 1;
        END WHILE;
      ELSE
        WHILE min_month <= 12 DO
          CALL copy_for_month(remove_archives, min_year, min_month, live_table_name, @temp_table_name);
          SET min_month = min_month + 1;
        END WHILE;
    END IF;
    SET min_year = min_year + 1;
    SET min_month = 1;
  END WHILE;
  
  SET @query = concat("DROP TABLE IF EXISTS ", @temp_table_name);
    
  /*SELECT @query;*/
  
  PREPARE sqlquery FROM @query;
  EXECUTE sqlquery;
  DEALLOCATE PREPARE sqlquery;
  
  SELECT concat(CURDATE(), " ", CURTIME(), ": Finished sort") AS "Action";
END;
